#!/usr/bin/env python
# -*- coding: utf-8 -*-
import time,os
import numpy as np
from dronekit import connect, VehicleMode, LocationGlobalRelative
from pymavlink import mavutil
import time

dirname=os.path.dirname(os.path.abspath(__file__))
ext = '.csv'
# Set up option parsing to get connection string
import argparse
parser = argparse.ArgumentParser(description='Commands vehicle using vehicle.simple_goto.')
parser.add_argument('--connect',
                    help="Vehicle connection target string. If not specified, SITL automatically started and used.")
args = parser.parse_args()

connection_string = args.connect
sitl = None

# Start SITL if no connection string specified
if not connection_string:
    import dronekit_sitl
    sitl = dronekit_sitl.start_default()
    connection_string = sitl.connection_string()


# Connect to the Vehicle
print('Connecting to vehicle on: %s' % connection_string)
vehicle = connect(connection_string, wait_ready=True, rate=100, baud=1500000)

start_t=time.time()
cnt=0

#vehicle.parameters['SR0_EXTRA1']=90
vehicle.parameters['SERIAL1_BAUD']=1500000
vehicle.parameters['SERIAL2_BAUD']=1500000

def document_message(message):
    m=message.to_dict()
    execlude = ['mavpackettype']
    if not os.path.isfile(m['mavpackettype'] +ext):
        with open(m['mavpackettype']+ext, 'w') as f:
            print(*[str(k)+ ',' for k,v in m.items() if k not in  execlude], file=f)
    with open(m['mavpackettype']+ext, 'a') as f:
         print(*[str(v)+ ',' for k,v in m.items() if k not in  execlude], file=f) 
         
@vehicle.on_message('RAW_IMU')
def RAW_IMU(self, name, msg):
	document_message(msg)

@vehicle.on_message('*')
def listener(self, name, message):
    global cnt
    execlude = ['mavpackettype']
    m=message.to_dict()   
    
    if m['mavpackettype']=='HEARTBEAT': #every second        
        set_intervals(27,10000)#RAW_IMU  
   

def set_intervals(messageID,interval):
    msg = vehicle.message_factory.command_long_encode(
	1, 0, #target system, target component
	mavutil.mavlink.MAV_CMD_SET_MESSAGE_INTERVAL, #command
	0, #confirmation
	messageID, #param 1
	interval, #param 2
	0, 0, 0, 0, 0) #param 3-7 not used
    vehicle.send_mavlink(msg)
    
def send_odom_from_gps(ned, velned):
    msg = vehicle.message_factory.odometry_encode(
            0,
            20,12, #changed to 20 to MAV_FRAME_LOCAL_FRD istead of MAV_FRAME_VISION_NED(deprecated)
            ned[0],ned[1],ned[2],
            [1,0,0,0],
            velned[0],velned[1],velned[2],
            0,0,0,
            [ ### mavlink2 use MAVLINK20=
                3.0,    0.0,    0.0,    0.0,    0.0,    0.0,
                        3.0,    0.0,    0.0,    0.0,    0.0,
                                20.0,  0.0,    0.0,    0.0,
                                        inf,    0.0,    0.0,
                                                inf,    0.0,
                                                        inf],
            [ ### mavlink2 use MAVLINK20=
                0.2,    0.0,    0.0,        0.0,    0.0,    0.0,
                        0.2,    0.0,        0.0,    0.0,    0.0,
                                20.0**2,    0.0,    0.0,    0.0,
                                            inf,    0.0,    0.0,
                                                    inf,    0.0,
                                                            inf],
            gs.estimate_reset_cnt,
            4, #https://mavlink.io/en/messages/common.html#MAV_ESTIMATOR_TYPE
            )
    vehicle.send_mavlink(msg)
    vehicle.flush()



def arm_and_takeoff(aTargetAltitude):
    """
    Arms vehicle and fly to aTargetAltitude.
    """

    print("Basic pre-arm checks")
    # Don't try to arm until autopilot is ready
    while not vehicle.is_armable:
        print(" Waiting for vehicle to initialise...")
        time.sleep(1)

    print("Arming motors")
    # Copter should arm in GUIDED mode
    vehicle.mode = VehicleMode("GUIDED")
    vehicle.armed = True

    # Confirm vehicle armed before attempting to take off
    while not vehicle.armed:
        print(" Waiting for arming...")
        time.sleep(1)

    print("Taking off!")
    vehicle.simple_takeoff(aTargetAltitude)  # Take off to target altitude

    # Wait until the vehicle reaches a safe height before processing the goto
    #  (otherwise the command after Vehicle.simple_takeoff will execute
    #   immediately).
    while True:
        print(" Altitude: ", vehicle.location.global_relative_frame.alt)
        # Break and return from function just below target altitude.
        if vehicle.location.global_relative_frame.alt >= aTargetAltitude * 0.95:
            print("Reached target altitude")
            break
        time.sleep(1)

arm_and_takeoff(10)

print("Set default/target airspeed to 3")
vehicle.airspeed = 3

print("Going towards first point for 30 seconds ...")
point1 = LocationGlobalRelative(-35.361354, 149.165218, 20)
vehicle.simple_goto(point1)

# sleep so we can see the change in map
time.sleep(30)

print("Going towards second point for 30 seconds (groundspeed set to 10 m/s) ...")
point2 = LocationGlobalRelative(-35.363244, 149.168801, 20)
vehicle.simple_goto(point2, groundspeed=10)

# sleep so we can see the change in map
time.sleep(30)

print("Returning to Launch")
vehicle.mode = VehicleMode("RTL")

# Close vehicle object before exiting script
print("Close vehicle object")
vehicle.close()

# Shut down simulator if it was started.
#if sitl:
#    sitl.stop()
